Changes zfs zpool initial / desired ashift to be based off stripesize
instead of sectorsize making it compatible with drives marked with
the 4k sector size quirk.

Without the correct min block size BIO_DELETE requests passed to
a large number of current SSD's via TRIM don't actually perform
any LBA TRIM so its vital for the correct operation of TRIM to get
the correct min block size.

To do this we added the additional dashift (desired ashift) to
vdev_open_func_t calls. This was needed as just updating ashift to
be based off stripesize would mean that a devices reported minimum
transfer size (ashift) could increase and that in turn would cause
member devices to be unusable and hence break pools with error
ZFS-8000-5E.

The global minimum ashift used for new zpools can now also be
tuned using the vfs.zfs.min_create_ashift sysctl. This defaults
to 12 (4096 byte blocks) in order to optimise for newer disks which
are migrating from 512 to 4096 byte sectors.

The value of vfs.zfs.min_create_ashift is limited to min of
SPA_MINBLOCKSHIFT (9) and a max of SPA_MAXBLOCKSHIFT (17).
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_disk.c.orig	2011-06-06 09:36:46.000000000 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_disk.c	2012-11-02 14:47:55.293668071 +0000
@@ -32,6 +32,8 @@
 #include <sys/sunldi.h>
 #include <sys/fm/fs/zfs.h>
 
+extern int zfs_min_ashift;
+
 /*
  * Virtual device vector for disks.
  */
@@ -103,7 +105,7 @@
 }
 
 static int
-vdev_disk_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift)
+vdev_disk_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift, uint64_t *dashift)
 {
 	spa_t *spa = vd->vdev_spa;
 	vdev_disk_t *dvd;
@@ -284,7 +286,7 @@
 	}
 
 	/*
-	 * Determine the device's minimum transfer size.
+	 * Determine the device's minimum and desired transfer size.
 	 * If the ioctl isn't supported, assume DEV_BSIZE.
 	 */
 	if (ldi_ioctl(dvd->vd_lh, DKIOCGMEDIAINFOEXT, (intptr_t)&dkmext,
@@ -292,6 +294,7 @@
 		dkmext.dki_pbsize = DEV_BSIZE;
 
 	*ashift = highbit(MAX(dkmext.dki_pbsize, SPA_MINBLOCKSIZE)) - 1;
+	*dashift = highbit(MAX(dkmext.dki_pbsize, (1ULL << zfs_min_ashift))) - 1;
 
 	/*
 	 * Clear the nowritecache bit, so that on a vdev_reopen() we will
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_file.c.orig	2012-01-05 22:31:25.000000000 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_file.c	2012-11-02 14:47:38.252107541 +0000
@@ -30,6 +30,8 @@
 #include <sys/fs/zfs.h>
 #include <sys/fm/fs/zfs.h>
 
+extern int zfs_min_ashift;
+
 /*
  * Virtual device vector for files.
  */
@@ -47,7 +49,7 @@
 }
 
 static int
-vdev_file_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift)
+vdev_file_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift, uint64_t *dashift)
 {
 	vdev_file_t *vf;
 	vnode_t *vp;
@@ -127,6 +129,7 @@
 
 	*psize = vattr.va_size;
 	*ashift = SPA_MINBLOCKSHIFT;
+	*dashift = zfs_min_ashift;
 
 	return (0);
 }
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_geom.c.orig	2012-11-02 12:20:15.918986181 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_geom.c	2012-11-02 14:47:48.135273692 +0000
@@ -36,6 +36,8 @@
 #include <geom/geom.h>
 #include <geom/geom_int.h>
 
+extern int zfs_min_ashift;
+
 /*
  * Virtual device vector for GEOM.
  */
@@ -408,7 +410,7 @@
 }
 
 static int
-vdev_geom_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift)
+vdev_geom_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift, uint64_t *dashift)
 {
 	struct g_provider *pp;
 	struct g_consumer *cp;
@@ -494,9 +496,10 @@
 	*psize = pp->mediasize;
 
 	/*
-	 * Determine the device's minimum transfer size.
+	 * Determine the device's minimum and desired transfer size.
 	 */
 	*ashift = highbit(MAX(pp->sectorsize, SPA_MINBLOCKSIZE)) - 1;
+	*dashift = highbit(MAX(pp->stripesize, (1ULL << zfs_min_ashift))) - 1;
 
 	/*
 	 * Clear the nowritecache settings, so that on a vdev_reopen()
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_mirror.c.orig	2012-07-03 11:49:22.342245151 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_mirror.c	2012-07-03 11:58:02.161948585 +0000
@@ -127,7 +127,7 @@
 }
 
 static int
-vdev_mirror_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift)
+vdev_mirror_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift, uint64_t *dashift)
 {
 	int numerrors = 0;
 	int lasterror = 0;
@@ -150,6 +150,7 @@
 
 		*asize = MIN(*asize - 1, cvd->vdev_asize - 1) + 1;
 		*ashift = MAX(*ashift, cvd->vdev_ashift);
+		*dashift = MAX(*dashift, cvd->vdev_dashift);
 	}
 
 	if (numerrors == vd->vdev_children) {
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_missing.c.orig	2012-07-03 11:49:10.545275865 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_missing.c	2012-07-03 11:58:07.670470640 +0000
@@ -40,7 +40,7 @@
 
 /* ARGSUSED */
 static int
-vdev_missing_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift)
+vdev_missing_open(vdev_t *vd, uint64_t *psize, uint64_t *ashift, uint64_t *dashift)
 {
 	/*
 	 * Really this should just fail.  But then the root vdev will be in the
@@ -50,6 +50,7 @@
 	 */
 	*psize = 0;
 	*ashift = 0;
+	*dashift = 0;
 	return (0);
 }
 
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_raidz.c.orig	2012-07-03 11:49:03.675875505 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_raidz.c	2012-07-03 11:58:15.334806334 +0000
@@ -1447,7 +1447,7 @@
 }
 
 static int
-vdev_raidz_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift)
+vdev_raidz_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift, uint64_t *dashift)
 {
 	vdev_t *cvd;
 	uint64_t nparity = vd->vdev_nparity;
@@ -1476,6 +1476,7 @@
 
 		*asize = MIN(*asize - 1, cvd->vdev_asize - 1) + 1;
 		*ashift = MAX(*ashift, cvd->vdev_ashift);
+		*dashift = MAX(*dashift, cvd->vdev_dashift);
 	}
 
 	*asize *= vd->vdev_children;
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_root.c.orig	2012-07-03 11:49:27.901760380 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_root.c	2012-07-03 11:58:19.704427068 +0000
@@ -50,7 +50,7 @@
 }
 
 static int
-vdev_root_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift)
+vdev_root_open(vdev_t *vd, uint64_t *asize, uint64_t *ashift, uint64_t *dashift)
 {
 	int lasterror = 0;
 	int numerrors = 0;
@@ -78,6 +78,7 @@
 
 	*asize = 0;
 	*ashift = 0;
+	*dashift = 0;
 
 	return (0);
 }
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev.c.orig	2012-10-22 20:41:50.234005351 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev.c	2012-10-22 20:42:16.355805894 +0000
@@ -1125,6 +1125,7 @@
 	uint64_t osize = 0;
 	uint64_t asize, psize;
 	uint64_t ashift = 0;
+	uint64_t dashift = 0;
 
 	ASSERT(vd->vdev_open_thread == curthread ||
 	    spa_config_held(spa, SCL_STATE_ALL, RW_WRITER) == SCL_STATE_ALL);
@@ -1154,7 +1155,7 @@
 		return (ENXIO);
 	}
 
-	error = vd->vdev_ops->vdev_op_open(vd, &osize, &ashift);
+	error = vd->vdev_ops->vdev_op_open(vd, &osize, &ashift, &dashift);
 
 	/*
 	 * Reset the vdev_reopening flag so that we actually close
@@ -1255,14 +1256,16 @@
 		 */
 		vd->vdev_asize = asize;
 		vd->vdev_ashift = MAX(ashift, vd->vdev_ashift);
+		vd->vdev_dashift = MAX(dashift, vd->vdev_dashift);
 	} else {
 		/*
 		 * Make sure the alignment requirement hasn't increased.
 		 */
 		if (ashift > vd->vdev_top->vdev_ashift) {
+			printf("ZFS ashift open failure of %s (%ld > %ld)\n", vd->vdev_path, ashift, vd->vdev_top->vdev_ashift);
 			vdev_set_state(vd, B_TRUE, VDEV_STATE_CANT_OPEN,
 			    VDEV_AUX_BAD_LABEL);
 			return (EINVAL);
 		}
 	}
 
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_label.c.orig	2012-11-05 15:27:52.092194343 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/vdev_label.c	2012-11-05 15:53:26.449021023 +0000
@@ -145,9 +145,12 @@
 #include <sys/fs/zfs.h>
 
 static boolean_t vdev_trim_on_init = B_TRUE;
+static boolean_t vdev_dashift_enable = B_TRUE;
 SYSCTL_DECL(_vfs_zfs_vdev);
 SYSCTL_INT(_vfs_zfs_vdev, OID_AUTO, trim_on_init, CTLFLAG_RW,
     &vdev_trim_on_init, 0, "Enable/disable full vdev trim on initialisation");
+SYSCTL_INT(_vfs_zfs_vdev, OID_AUTO, optimal_ashift, CTLFLAG_RW,
+    &vdev_dashift_enable, 0, "Enable/disable optimal ashift usage on initialisation");
 
 /*
  * Basic routines to read and write from a vdev label.
@@ -282,6 +285,16 @@
 		    vd->vdev_ms_array) == 0);
 		VERIFY(nvlist_add_uint64(nv, ZPOOL_CONFIG_METASLAB_SHIFT,
 		    vd->vdev_ms_shift) == 0);
+		/*
+		 * We use the max of ashift and dashift (the desired/optimal
+		 * ashift), which is typically the stripesize of a device, to
+		 * ensure we get the best performance from underlying devices.
+		 * 
+		 * Its done here as it should only ever have an effect on new
+		 * zpool creation.
+		 */
+		if (vdev_dashift_enable)
+			vd->vdev_ashift = MAX(vd->vdev_ashift, vd->vdev_dashift);
 		VERIFY(nvlist_add_uint64(nv, ZPOOL_CONFIG_ASHIFT,
 		    vd->vdev_ashift) == 0);
 		VERIFY(nvlist_add_uint64(nv, ZPOOL_CONFIG_ASIZE,
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/sys/vdev_impl.h.orig	2012-10-22 20:40:08.361577293 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/sys/vdev_impl.h	2012-10-22 21:02:52.447781800 +0000
@@ -55,7 +55,7 @@
 /*
  * Virtual device operations
  */
-typedef int	vdev_open_func_t(vdev_t *vd, uint64_t *size, uint64_t *ashift);
+typedef int	vdev_open_func_t(vdev_t *vd, uint64_t *size, uint64_t *ashift, uint64_t *dashift);
 typedef void	vdev_close_func_t(vdev_t *vd);
 typedef uint64_t vdev_asize_func_t(vdev_t *vd, uint64_t psize);
 typedef int	vdev_io_start_func_t(zio_t *zio);
@@ -119,6 +119,7 @@
 	uint64_t	vdev_asize;	/* allocatable device capacity	*/
 	uint64_t	vdev_min_asize;	/* min acceptable asize		*/
 	uint64_t	vdev_ashift;	/* block alignment shift	*/
+	uint64_t	vdev_dashift;	/* desired blk alignment shift	*/
 	uint64_t	vdev_state;	/* see VDEV_STATE_* #defines	*/
 	uint64_t	vdev_prevstate;	/* used when reopening a vdev	*/
 	vdev_ops_t	*vdev_ops;	/* vdev operations		*/
--- sys/cddl/contrib/opensolaris/uts/common/fs/zfs/dsl_pool.c.orig	2012-11-02 14:56:29.474248887 +0000
+++ sys/cddl/contrib/opensolaris/uts/common/fs/zfs/dsl_pool.c	2012-11-03 01:27:28.066912403 +0000
@@ -41,6 +41,30 @@
 #include <sys/spa_impl.h>
 #include <sys/dsl_deadlist.h>
 
+#define ZFS_MIN_ASHIFT SPA_MINBLOCKSHIFT
+/*
+ * Max ashift - limited by how labels are accessed by zio_read_phys using offsets
+ * within vdev_label_t
+ *
+ * If label access is fixed to work with ashift properly then the max should be
+ * set to SPA_MAXBLOCKSHIFT
+ */
+#define ZFS_MAX_ASHIFT 13
+/*
+ * Optimum ashift - defaults to 12 which results in a min block size of 4096 as
+ * this is the optimum value for newer disks which are migrating from 512 to 4096
+ * byte sectors
+ */
+#define ZFS_OPTIMUM_ASHIFT 12 
+
+/*
+ * Minimum ashift used when creating new pools
+ *
+ * This can be tuned using the sysctl vfs.zfs.min_create_ashift but is limited
+ * to a min of ZFS_MIN_ASHIFT and a max of ZFS_MAX_ASHIFT
+ * 
+ */
+int zfs_min_ashift = MAX(SPA_MINBLOCKSHIFT, ZFS_OPTIMUM_ASHIFT);
 int zfs_no_write_throttle = 0;
 int zfs_write_limit_shift = 3;			/* 1/8th of physical memory */
 int zfs_txg_synctime_ms = 1000;		/* target millisecs to sync a txg */
@@ -54,6 +78,9 @@
 
 static pgcnt_t old_physmem = 0;
 
+#ifdef _KERNEL
+static int min_ashift_sysctl(SYSCTL_HANDLER_ARGS);
+
 SYSCTL_DECL(_vfs_zfs);
 TUNABLE_INT("vfs.zfs.no_write_throttle", &zfs_no_write_throttle);
 SYSCTL_INT(_vfs_zfs, OID_AUTO, no_write_throttle, CTLFLAG_RDTUN,
@@ -78,6 +105,32 @@
 TUNABLE_QUAD("vfs.zfs.write_limit_override", &zfs_write_limit_override);
 SYSCTL_QUAD(_vfs_zfs, OID_AUTO, write_limit_override, CTLFLAG_RDTUN,
     &zfs_write_limit_override, 0, "");
+SYSCTL_PROC(_vfs_zfs, OID_AUTO, min_create_ashift, CTLTYPE_INT | CTLFLAG_RW,
+    &zfs_min_ashift, 0, min_ashift_sysctl, "I",
+    "Minimum ashift used when creating new pools");
+
+static int
+min_ashift_sysctl(SYSCTL_HANDLER_ARGS)
+{
+	int error, value;
+
+	value = *(int *)arg1;
+
+	error = sysctl_handle_int(oidp, &value, 0, req);
+
+	if ((error != 0) || (req->newptr == NULL))
+		return (error);
+
+	if (value < ZFS_MIN_ASHIFT)
+		value = ZFS_MIN_ASHIFT;
+	else if (value > ZFS_MAX_ASHIFT)
+		value = ZFS_MAX_ASHIFT;
+
+	*(int *)arg1 = value;
+
+	return (0);
+}
+#endif
 
 int
 dsl_pool_open_special_dir(dsl_pool_t *dp, const char *name, dsl_dir_t **ddp)
